const CommonSelectors = require('../lib/common_selectors');
const doubleClickZoom = require('../lib/double_click_zoom');
const Constants = require('../constants');

const DrawCircle = {};

DrawCircle.onSetup = function() {
  const currentClickNum = 0;
  const center = {
    x:0,
    y:0
  };
  // eslint-disable-next-line no-console
  const circle = this.newFeature({
    type: Constants.geojsonTypes.FEATURE,
    properties: {
      type:Constants.geojsonTypes.CIRCLE,
      draw_type:"circle"
    },
    geometry: {
      type: Constants.geojsonTypes.POLYGON,
      coordinates: [[]]
    }
  }, Constants.geojsonTypes.CIRCLE);


  this.addFeature(circle);

  this.clearSelectedFeatures();
  doubleClickZoom.disable(this);
  this.updateUIClasses({ mouse: Constants.cursors.ADD });
  this.activateUIButton(Constants.types.POLYGON);
  this.setActionableState({
    combineFeatures: false,
    uncombineFeatures: false,
    trash: true
  });

  return {
    circle,
    currentClickNum,
    center
  };
};



DrawCircle.clickAnywhere = function(state, e) {
  /*if (currentVertexPosition > 0 && isEventAtCoordinates(e, circle.coordinates[0][currentVertexPosition - 1])) {
            return ctx.events.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [circle.id] });
          }*/
  this.updateUIClasses({ mouse: Constants.cursors.ADD });
  if (state.currentClickNum === 0) {
    state.center = this.map.project(e.lngLat);
    // eslint-disable-next-line no-const-assign
    state.currentClickNum++;
  } else if (state.currentClickNum === 1) {
    state.currentClickNum++;
  }
};


DrawCircle.clickOnVertex = function(state) {
  return this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.polygon.id] });
};

DrawCircle.onMouseMove = function(state, e) {
  if (state.currentClickNum === 1) {
    // eslint-disable-next-line no-undef
    const circleVertex = state.circle.getCircleVertex(this, state.center, this.map.project(e.lngLat));
    if (circleVertex) {
      state.circle.setCoordinates([circleVertex]);
    }
  } else if (state.currentClickNum === 2) { //结束
    debugger
    this.map.fire(Constants.events.CREATE, {
      features: [state.circle.toGeoJSON()]
    });
    this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.circle.id] });
  }
  if (CommonSelectors.isVertex(e)) {
    this.updateUIClasses({ mouse: Constants.cursors.POINTER });
  }
};


DrawCircle.onTap = DrawCircle.onClick = function(state, e) {
  if (CommonSelectors.isVertex(e)) return this.clickOnVertex(state, e);
  return this.clickAnywhere(state, e);
};




DrawCircle.onKeyUp = function(state, e) {
  if (CommonSelectors.isEscapeKey(e)) {
    this.deleteFeature([state.circle.id], { silent: true });
    this.changeMode(Constants.modes.SIMPLE_SELECT);
  } else if (CommonSelectors.isEnterKey(e)) {
    this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.circle.id] });
  }
};

DrawCircle.onStop = function(state) {
  this.updateUIClasses({ mouse: Constants.cursors.NONE });
  doubleClickZoom.enable(this);
  this.activateUIButton();

  // check to see if we've deleted this feature
  if (this.getFeature(state.circle.id) === undefined) return;

  /*//remove last added coordinate
  circle.removeCoordinate("0."+currentVertexPosition);*/
  if (state.circle.isValid()) {
    this.map.fire(Constants.events.CREATE, {
      features: [state.circle.toGeoJSON]
    });
  } else {
    this.deleteFeature([state.circle.id], { silent: true });
    this.changeMode(Constants.modes.SIMPLE_SELECT, {}, { silent: true });
  }
};

DrawCircle.toDisplayFeatures = function(state, geojson, display) {
  const isActivePolygon = geojson.properties.id === state.circle.id;
  geojson.properties.active = (isActivePolygon) ? Constants.activeStates.ACTIVE : Constants.activeStates.INACTIVE;
  if (!isActivePolygon) return display(geojson);

  // Don't render a polygon until it has two positions
  // (and a 3rd which is just the first repeated)
  if (geojson.geometry.coordinates.length === 0) return;

  const coordinateCount = geojson.geometry.coordinates[0].length;

  // If we have fewer than two positions (plus the closer),
  // it's not yet a shape to render
  if (coordinateCount < 3) return;

  geojson.properties.meta = Constants.meta.FEATURE;

  /*if (coordinateCount > 4) {
    // Add a start position marker to the map, clicking on this will finish the feature
    // This should only be shown when we're in a valid spot
    callback(createVertex(circle.id, geojson.geometry.coordinates[0][0], '0.0', false));
    var endPos = geojson.geometry.coordinates[0].length - 3;
    callback(createVertex(circle.id, geojson.geometry.coordinates[0][endPos], "0."+endPos, false));
  }*/

  // If we have more than two positions (plus the closer),
  // render the Polygon
  if (coordinateCount > 3) {
    return display(geojson);
  }

  // If we've only drawn two positions (plus the closer),
  // make a LineString instead of a Polygon
  const lineCoordinates = [
    [geojson.geometry.coordinates[0][0][0], geojson.geometry.coordinates[0][0][1]], [geojson.geometry.coordinates[0][1][0], geojson.geometry.coordinates[0][1][1]]
  ];
  return display({
    type: Constants.geojsonTypes.FEATURE,
    properties: geojson.properties,
    geometry: {
      coordinates: lineCoordinates,
      type: Constants.geojsonTypes.LINE_STRING
    }
  });
};

DrawCircle.onTrash  = function(state) {
  this.deleteFeature([state.circle.id], { silent: true });
  this.changeMode(Constants.modes.SIMPLE_SELECT);
};
module.exports = DrawCircle;
